#!/usr/bin/perl -w
#
# wm2mbox.pl
#
#   version 1.00 (2010-10-05)
#
#   Copyright (C) 2010 MURATA Yasuhisa. All rights reserved.
#
#   This program is free software; you can redistribute it and/or
#   modify it under the same terms as Perl itself.
#

use 5.008; use 5.8.5;

# untaint environment : cf. perldoc perlrun
$ENV{PATH}  = '/bin:/usr/bin';
$ENV{SHELL} = '/bin/sh' if exists $ENV{SHELL};
delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};

use warnings;
use strict;

use File::Find;
use Cwd;
use Encode qw/from_to/;
use Encode::JP; # for PAR

my $crlf = "\x0d\x0a";
my $ext = '.wem';
my $pref = 'mbox';

my $cwd = Cwd::getcwd();
my %find_dir;

find \&process_file, '.';

exit 0;


sub process_file {
    return unless -f;

    my $dir = $File::Find::dir;
    return if (defined $find_dir{$dir});
    $find_dir{$dir} = 1;

    print "$dir\n";

    my $file;
    my @mn;
    opendir(DIR, '.');
    while (defined($file = readdir(DIR))) {
        next unless ($file =~ s/(\d+)$ext$/$1/);
        push @mn, $file;
    }
    closedir(DIR);
    return if @mn == 0;

    for my $i (sort {$a <=> $b} @mn) {
        my $data;
        my %header;
        my $h = '';
        my $from;

        open(MAIL, "$i$ext") or die "cannot open $i$ext, $!\n";
        binmode(MAIL);

        while (<MAIL>) { # read header
            $data .= $_;

            s/\x0d?\x0a$//;
            last if (/^$/);
            if (/^\s+(.*)/) {
                $header{"\L$h\E"} .= " $1";
                next;
            }
            /^(.*?):\s*(.*)$/;
            $h = $1;
            $header{"\L$h\E"} = $2;
        }

        while (<MAIL>) { # read body
            s/^/>/ if /^From /;
            $data .= $_;
        }
        close(MAIL);

        unless (defined($header{'content-type'}) and
            $header{'content-type'} =~
            /^text\/plain\; charset\=shift_jis/i) {
            from_to($data, 'cp932', '7bit-jis');
        }

        if (defined($header{'return-path'})) {
            $from = $header{'return-path'};
            $from =~ s/^\<(.*)\>$/$1/;
        } else {
            $from = $header{'from'};
            $from = $1 if $from =~ /\<(\S+?)\>/;
            $from = $1 if $from =~ /(\S+)\s*\(.+\)/;
        }
        my ($wday, $mday, $mon, $year, $time, $tz)
            = split(/[,\s]+/,$header{'date'});
        if ($year < 50) {
            $year += 2000;
        }
        elsif ($year < 1900) {
            $year += 1900;
        }

        my $mbox = $dir;
        $mbox =~ s/^\.//;
        $mbox =~ s/\//_/g;

        open(OUT, '>>', "$cwd/$pref$mbox");
        binmode(OUT);
        print OUT "From $from $wday $mon $mday $time $year$crlf";
        print OUT $data;
        print OUT $crlf;
        close OUT;
    }
}
